﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Arction.WinForms.Charting;
using Arction.WinForms.Charting.SeriesXY;

namespace IntensityMeshTest
{
    public partial class Form1 : Form
    {        /// <summary>
             /// LightningChart component.
             /// </summary>
        private LightningChartUltimate _chart => lightningChartUltimate1;

        /// <summary>
        /// Intesity mesh.
        /// </summary>
        private IntensityMeshSeries _intensityMesh = null;

        /// <summary>
        /// Mesh size column count.
        /// </summary>
        private int _columns = 10;

        /// <summary>
        /// Mesh size row count.
        /// </summary>
        private int _rows = 10;

        public Form1()
        {
            InitializeComponent();

            CreateChart();
            CreateMeshGeometry();
            GenerateValues();
        }



        /// <summary>
        /// Create chart
        /// </summary>
        private void CreateChart()
        {
            //Disable rendering, strongly recommended before updating chart properties
            _chart.BeginUpdate();

            _chart.Dock = DockStyle.Fill;
            _chart.ActiveView = ActiveView.ViewXY;

            // Configure x-axis.
            _chart.ViewXY.XAxes[0].ValueType = AxisValueType.Number;
            _chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.None;
            _chart.ViewXY.XAxes[0].SetRange(-5, 105);

            // Configure y-axis.
            _chart.ViewXY.YAxes[0].SetRange(-5, 105);


            //// Create intensity mesh series.
            //_intensityMesh = new IntensityMeshSeries(_chart.ViewXY, _chart.ViewXY.XAxes[0], _chart.ViewXY.YAxes[0])
            //{
            //    ContourLineType = ContourLineTypeXY.None,
            //    Optimization = IntensitySeriesOptimization.DynamicValuesData,
            //    MouseInteraction = false,
            //    ShowNodes = false
            //};

            _intensityMesh = new IntensityMeshSeries(_chart.ViewXY, _chart.ViewXY.XAxes[0], _chart.ViewXY.YAxes[0])
            {
                ContourLineType = ContourLineTypeXY.ColorLine,
                MouseInteraction = false,
                ShowNodes = true,
                WireframeType = SurfaceWireframeType.None,
                Fill = IntensityFillStyle.Paletted,
                ContourLineStyle = { Color = Color.WhiteSmoke },
                ContourLineLabels = { Visible = true, Color = Color.WhiteSmoke },

                ShowInLegendBox = true,
                Title = { Text = "My Title" },
                LegendBoxUnits = String.Empty
            };


            _chart.ViewXY.IntensityMeshSeries.Add(_intensityMesh);

            //Create value range palette
            ValueRangePalette palette = new ValueRangePalette(_intensityMesh);
            //ExampleUtils.DisposeAllAndClear(palette.Steps); //Remove existing palette steps
            palette.MinValue = 0;
            palette.Type = PaletteType.Gradient;
            palette.Steps.Add(new PaletteStep(palette, Color.Blue, 10));
            palette.Steps.Add(new PaletteStep(palette, Color.Green, 30));
            palette.Steps.Add(new PaletteStep(palette, Color.Yellow, 80));
            palette.Steps.Add(new PaletteStep(palette, Color.Red, 150));
            _intensityMesh.ValueRangePalette = palette;

            // Configure legend.
            _chart.ViewXY.LegendBoxes[0].Position = LegendBoxPositionXY.SegmentBottomRight;

            // Allow chart rendering.
            _chart.EndUpdate();

            //base.RaiseChartsCreatedEvent();
        }

        /// <summary>
        /// Create mesh geometry when the animation is started.
        /// </summary>
        private void CreateMeshGeometry()
        {
            //Disable rendering, strongly recommended before updating chart properties
            _chart.BeginUpdate();

            //Create a new data array 
            int columns = _columns;
            int rows = _rows;
            IntensityPoint[,] data = new IntensityPoint[columns, rows];

            double minX = 0;
            double maxX = 100;
            double yMin = 0;
            double yMax = 100;

            double totalX = maxX - minX;
            double totalY = yMax - yMin;
            double stepX = totalX / (double)columns;
            double stepY = totalY / (double)rows;

            double bendAmplitude = (double)2;


            Random rnd = new Random();

            double y = yMin;
            for (int row = 0; row < rows; row++)
            {
                double x = minX;
                for (int col = 0; col < rnd.Next(6, columns); col++)
                //for (int col = 0; col < columns; col++)
                {
                    data[col, row].X = x + bendAmplitude * Math.Sin((double)row / 16.0);
                    data[col, row].Y = y + bendAmplitude * Math.Cos((double)col / 16.0);

                    x += stepX;
                }
                y += stepY;
            }

            _intensityMesh.Data = data;


            //Allow chart rendering
            _chart.EndUpdate();
        }

        /// <summary>
        /// Generate new Values data for the mesh. Do not update geometry here.
        /// </summary>
        private void GenerateValues()
        {
            int columns = _columns;
            int rows = _rows;

            //Disable rendering, strongly recommended before updating chart properties
            _chart.BeginUpdate();

            double minX = 0;
            double maxX = 100;
            double yMin = 0;
            double yMax = 100;

            double dValueMin = 0;
            double dValueMax = 100;

            double totalX = maxX - minX;
            double totalY = yMax - yMin;
            double stepX = totalX / (double)columns;
            double stepY = totalY / (double)rows;

            double centerX = (maxX + minX) / 2.0;

            IntensityPoint[,] data = _intensityMesh.Data;

            double y = yMin;
            for (int row = 0; row < rows; row++)
            {
                double x = minX;
                for (int col = 0; col < columns; col++)
                {
                    var sideFactor = 1.0 - 2.0 * ((centerX - x) / totalX) * ((centerX - x) / totalX);

                    data[col, row].Value = (dValueMax + dValueMin) / 2.0 + 1 / 2.0 * Math.Sin(x / 8.0) * Math.Sin(y / 8.0) * sideFactor + y / 2;

                    x += stepX;
                }
                y += stepY;
            }

            //Invalidate series data only
            _intensityMesh.InvalidateValuesDataOnly();

            //Allow chart rendering
            _chart.EndUpdate();
        }
    }
}
